-- La table est le seul type de structure en Lua.

-- Une table doit toujours être construite avec des accolades
-- avant de pouvoir être utilisée.
a = {} -- Les accolades représentent le *constructeur*
print(type(a), a) --> table  table: 0x9abcdef0

-- On utilise des crochets pour indexer une table en écriture
a[1] = "un"
a[2] = "deux"
-- Les crochets sont utilisés de même en lecture.
-- Si la valeur n'est pas dans la table, nil est retourné.
print(a[1], a[2], a[3]) --> un  deux  nil

-- Dans le constructeur, il est possible de donner des valeurs
-- initiales. Si les valeurs sont simplement listées par des
-- virgules, par convention les valeurs d'indices sont les
-- nombres entiers à partir de 1 :
a = { "one", "two", "three" }
print(a[1], a[2], a[3]) --> one two three

-- On peut utiliser toute valeur comme indice (sauf nil et NaN)
a["chaine"] = "valeur"
a[2+1] = "trois"
a[true] = 42
a[0.5] = false
a[print] = "une fonction" -- même une fonction
print(a['chaine'], a[not false], a[1/2]) --> valeur  42  false

-- La fonction pairs permet de parcourir l'ensemble des paires
-- indices / valeurs, dans une boucle for avec une syntaxe
-- un peu particulière :
for index,value in pairs(a) do
  print(index, value)  -- affichera les 7 entrées de la table
end

-- Le point permet de simplifier l'accès d'une table si l'indice
-- est une chaîne constante assimilable à un nom de variable,
-- c'est-à-dire t.nom == t["nom"] :
print(a.chaine) --> valeur
a.unNom = 33
print(a["unNom"]) --> 33

-- L'opérateur # retourne la longueur d'une table. Elle est définie comme le plus
-- grand nombre entier N tel que t[N] n'est pas nil.
-- ATTENTION: il ne faut pas qu'il y ait de "trous" dans la séquence !
print(#a) --> 3

-- Enfin, dans un constructeur, il est possible de spécifier
-- n'importe quel indice. On aurait pu construire la table
-- précédente directement avec cette syntaxe :
a = { "un", "deux", "trois", -- indices 1, 2 et 3.
  chaine = "valeur", unNom = 3, -- pour les mêmes indices que "."
  [true] = 42, [0.5] = false, [print] = "une fonction" } -- cas général
