local ffi = require 'ffi'
ffi.cdef [[
typedef unsigned long uLong, uLongf;
typedef unsigned char Bytef;

int compress2 (Bytef *dest, uLongf *destLen,
      const Bytef *source, uLong sourceLen, int level);

int uncompress (Bytef *dest, uLongf *destLen,
      const Bytef *source, uLong sourceLen);
]]
local zlib = ffi.load("z")
local function compress_common(data, fct, params)
  local buflen = #data * 2
  while true do
    local dest = ffi.new("Bytef[?]", buflen)
    local destLen = ffi.new("uLongf[1]", buflen)
    local res = fct(dest, destLen, data, #data, unpack(params))
    if res == 0 then
      return ffi.string(dest, destLen[0])
    end
    assert(res == -5, "zlib error")
    buflen = buflen * 2
  end
end
local function compress(data, level)
  return compress_common(data, zlib.compress2, {level or -1})
end
local function uncompress(data, level)
  return compress_common(data, zlib.uncompress, {})
end

-- Test
local orig = ("Hello World!\n"):rep(10000)
local compr = compress(orig)
print(#orig, #compr) --> 130000  293
local ucompr = uncompress(compr)
assert(ucompr == orig)
